/* users, groups and roles settings */
#form-ldap_users_settings .dictwidget th.actions,
#form-ldap_users_settings .dictwidget td.actions,
#form-ldap_groups_settings .dictwidget th.actions,
#form-ldap_groups_settings .dictwidget td.actions,
#form-ldap_roles_settings .dictwidget th.actions,
#form-ldap_roles_settings .dictwidget td.actions {
    width:7em;
    padding:5px 0 0 0;
}
#form-ldap_users_settings .dict_actions,
#form-ldap_groups_settings .dict_actions,
#form-ldap_roles_settings .dict_actions {
    margin-top:8px;
    padding-right:0;
    width:7em;
}
#form-ldap_users_settings tr th:first-child,
#form-ldap_users_settings tr td:first-child,
#form-ldap_groups_settings tr th:first-child,
#form-ldap_groups_settings tr td:first-child,
#form-ldap_roles_settings tr th:first-child,
#form-ldap_roles_settings tr td:first-child {
    padding-left:0;
}
#field-ldap_users_settings-users_aliases_attrmap label,
#field-ldap_groups_settings-groups_aliases_attrmap label,
#field-ldap_roles_settings-roles_aliases_attrmap label {
    font-size:130%;
    text-decoration:underline;
    margin-top:12px;
}
